<?php

require_once("nusoap/nusoap.php");
require_once("config/config.php");
require_once("VC/vCenter.php");
require_once("VC/vAPI.php");
require_once("VC/pc.php");
require_once("VC/si.php");
require_once("VC/sql.php");
require_once("VC/utils.php");
require_once("VC/dvswitch.php");
require_once("switch/raslog.php");
require_once("switch/portprofile.php");
require_once("switch/interface.php");
require_once("switch/logging.php");
require_once("switch/logconfig.php");

$ns = '';

define("SUCCESS",             0);
define("DISCOVER_NOTSET",     1);
define("INVALID_CONNECTION",  2);
define("INVALID_CREDENTIALS", 3);
define("INVALID_VERSION",     4);
define("NO_SERVICE",          5);
define("GENERIC_ERROR",     999);

$vcenter        = $_SERVER[argv][1];
$url            = $_SERVER[argv][2];
$user           = $_SERVER[argv][3];
$pass           = $_SERVER[argv][4];
//$timeout        = $_SERVER[argv][5];
        
$sw = new StopWatch();
$csw = new StopWatch();

$db = "sqlite:/tmp/dc.db.new";

$dbh = new DB($db);

$log = Logging::getInstance(); 

raslogv(RASLOG_INFO, "START: discover virtual assets from vCenter @ $url");

if ($activate !== "discover") {
        raslogv(RASLOG_WARNING, "discovery not allowed yet DISCOVER_NOTSET");
        $log->lclose();
        exit(DISCOVER_NOTSET);
}

if (($vc  = new vCenter($url, $ns, $user, $pass)) < 0) {
}

$ret = $vc->setDebug(0);
//$ret = $vc->setDebug($debug);

//if (($ret = $vc->vConnect($timeout)) < 0) {
if (($ret = $vc->vConnect()) < 0) {
        raslogv(RASLOG_WARNING, "Connect to vCenter failed (url = $url) INVALID_CONNECTION");
        $log->lclose();
        exit(INVALID_CONNECTION);
}

$ret = $vc->initServices();
if ($ret == -1) {
        raslogv(RASLOG_ERROR, "Cannot Initialize Services on vCenter");
        $log->lclose();
        exit(NO_SERVICE);
}
$version = $vc->getVersion();
        raslogv(VCLOG_INFO, "VC version $version");
if (!starts_with($version,"4") && !starts_with($version,"5")) {
        raslogv(RASLOG_ERROR, "Invalid vCenter Version ($version), (only versions 4.x and 5.x supported)");
        $log->lclose();
        exit(INVALID_VERSION);
}
if (($ret = $vc->login()) != 1) {
        raslogv(RASLOG_ERROR, "Failed communicating to vcenter $url Invalid Credentials ");
        $log->lclose();
        exit(INVALID_CREDENTIALS);
}
$time = $sw->elapsedTime();
raslogv(VCLOG_INFO, "Login succeed for user ($url) ");
raslogv(VCLOG_INFO, "Login TIME $time seconds ");


$sw->start();
try {


raslogv(VCLOG_INFO, "Fetching HostSystems");
$host_mors    = getMORS($vc, 'HostSystem');
raslogv(VCLOG_DEBUG, $host_mors);
foreach ($host_mors as $host_mor) {
        if(empty($host_mor)) continue;
        $host = new Host($vc, $host_mor);
        //$ret  = $host->retrieveProperties("summary.config");
        $ret  = $host->retrieveProperties();
        if ($ret == 0) {
            raslogv(VCLOG_WARNING, "SKIPPING host ".$host_mor);
            continue;
        }

        $host_nn = $host->getHostFQDN();
        $version = $host->getVersion();
        raslogv(VCLOG_INFO, "Processing host ".$host_nn." version ".$version);
        if (!starts_with($version,"4") && !starts_with($version,"5")) {
            raslogv(RASLOG_ERROR, "Invalid vCenter Version ($version), (only versions 4.x and 5.x supported)");
/*        if (($version != "4.0.0") && ($version != "4.1.0")) {
           raslogv(RASLOG_ERROR, "Invalid ESX Version ($version), (only versions 4.0.0 and 4.1.0 supported)");
*/
           continue;
        }
        //$host_nn = $host->getNN();
        $dbh->addObj($host_mor, $host_nn, "");

        //$ret  = $host->retrieveProperties("config.network");

        // VMNICS aka pnics per host 
        $vmnics   = $host->getPnics();

        foreach ($vmnics as $vmnic) {
                
                $vmnic_key = $vmnic['key'];
                $vmnic_nn = $vmnic['device'];
                $mac = $vmnic['mac'];
                raslogv(VCLOG_DEBUG,"Processing vmnics $host_mor $vmnic_key, $vmnic_nn $mac ");
                $attr[] = "host_id";$val[] = $host_mor;
                $vmnic_mor = "vmnic-" . $host_mor . "-".$vmnic_key;
                //$vmnic_mor = "vmnic-" . "$count";
                $dbh->addObj($vmnic_mor, $vmnic_nn, "");

                unset($attr); unset($val);
                $attr[] = "name";   $val[] = $vmnic_nn;
                $attr[] = "mac";   $val[] = $mac;
                $attr[] = "host_id";$val[] = $host_mor;
                $dbh->setAttr($vmnic_mor, $attr, $val);
        }
        // Vswitch per host 
        $vss   = $host->getvSwitch();
        unset($vs_names);
        $dbh->createRelation("portgroups-vs");

        foreach ($vss as $vs) {

                $vs_key = $host_mor.$vs['key'];
                $vs_nn = $vs['name'];

                raslogv(VCLOG_DEBUG,"Processing vswitch  $host_mor $vs_nn");
                $vs_mor = "vs-" . $vs_key;
                $dbh->addObj($vs_mor, $vs_nn, "");

                unset($attr); unset($val);
                $attr[] = "name";   $val[] = $vs_nn;
                $attr[] = "host";   $val[] = $host_nn;
                $attr[] = "host_id";$val[] = $host_mor;
                $dbh->setAttr($vs_mor, $attr, $val);

                //add vswitch info to vmnics table
                unset($pnics_temp);
                unset($pnics);
                $pnics_temp = $vs['pnic'];
                if (!is_array($pnics_temp)) {
                    $pnics[0] = $pnics_temp;
                } else {
                    $pnics = $pnics_temp;
                }
                //there can be multiple pnics per vswitch
                foreach ($pnics as $pnic) {
                    $vmnic_mor = "vmnic-" . $host_mor . "-".$pnic;
                    unset($attr); unset($val);
                    $attr[] = "dvs";   $val[] = $vs_mor;
                    $attr[] = "dvs_nn";   $val[] = $vs_nn;
                    $dbh->setAttr($vmnic_mor, $attr, $val);
                }

                $pg_keys_temp = $vs['portgroup'];
                if (!is_array($pg_keys_temp)) {
                    $pg_keys[] = $pg_keys_temp;
                } else {
                    $pg_keys = $pg_keys_temp;
                }
                $dbh->addRelations("portgroups-vs", $pg_keys, $vs_key);
                $vs_keys[] = $vs_key;

        }
        $dbh->createRelation("vss-host");
        $dbh->addRelations("vss-host", $vs_keys, $host_mor);

        $pgs = $host->getPortGroup();
        $pg_key   = $pgs['key'];
        // in case of only on pg a non array is returned
        if (!empty($pg_key)) {
            parseAPortgroup($host_nn,$host_mor, $pgs, $dbh);
        }else {
            foreach ($pgs as $pg) {
                parseAPortgroup($host_nn,$host_mor, $pg, $dbh);
            }
        }

        // DV Switch 
        $px   = $host->getProxySwitch();
        if (!empty($px)) {
        $dvs_name = $px['dvsName'];
        $backing = $px['spec']['backing'];
        if (is_array($backing)) {
                $vmnic    = $px['spec']['backing']['pnicSpec']['pnicDevice'];
                $upg_mor  = $px['spec']['backing']['pnicSpec']['uplinkPortgroupKey'];
        }

        unset($attr); unset($val);
        $attr[] = "name";    $val[] = $host_nn;
        //$attr[] = "vmnic";   $val[] = $vmnic;
        //$attr[] = "dvs";     $val[] = $dvs_name;
        $attr[] = "upg_mor"; $val[] = $upg_mor;
        $dbh->setAttr($host_mor, $attr, $val);
        }

        //$ret  = $host->retrieveProperties("vm");
        $hvm_mors = $host->getVMS();
        // populate table vms with id and host_id
        if (!empty($hvm_mors)) {
            unset($attr); unset($val); 
            foreach ($hvm_mors as $hvm_mor) {
        //raslogv(VCLOG_DEBUG,$hvm_mor);
                raslogv(VCLOG_DEBUG,"Processing VM  $host_mor $hvm_mor");
                $dbh->addObj($hvm_mor, "test");
                $attr[] = "host_id"; $val[] = "$host_mor";
                $attr[] = "host_nn"; $val[] = "$host_nn";
                $dbh->setAttr($hvm_mor, $attr, $val);
            }
        }
        
        $vmk_nics = $host->getVnics();
        if(empty($vmk_nics)) continue;
//      print_r($vmk_nics);
        foreach ($vmk_nics as $vmk) {
            if(empty($vmk)) continue;

            $vmk_mac = $vmk['spec']['mac'];
            $vmk_mor = "vnic-".$vmk_mac;
            raslogv(VCLOG_DEBUG, "Parsing VMKNICs $vmk_mor");
            $vmk_pg_nn = $vmk['portgroup'];
            $dbh->addObj($vmk_mor, "");
            
            if (!empty($vmk_pg_nn)) {
                $vmk_pg_id = "portgroup-".$host_mor."-key-vim.host.PortGroup-".urldecode($vmk_pg_nn);
            } else {
                $vmk_pg_id = $vmk['spec']['distributedVirtualPort']['portgroupKey'];
            }
            unset($attr); unset($val);
            $attr[] = "mac";     $val[] = $vmk_mac;
            $attr[] = "dvpg_id"; $val[] = $vmk_pg_id;
            $attr[] = "dvpg_nn"; $val[] = $vmk_pg_nn;
            $attr[] = "vm_id";   $val[] = "VMKernel";
            $dbh->setAttr($vmk_mor, $attr, $val);
        }
}
   $time = $sw->elapsedTime();
   raslogv(VCLOG_INFO, "Parsing Host-MOR TIME $time seconds ");

$sw->resetAndStart();

raslogv(VCLOG_INFO, "Fetching DVS");
$dvs_mors     = getMORS($vc, 'DistributedVirtualSwitch');
raslogv(VCLOG_DEBUG, $dvs_mors);
foreach ($dvs_mors as $dvs_mor) {
        if(empty($dvs_mor)) continue;
        $dvs = new dvSwitch($vc, $dvs_mor);
        $ret  = $dvs->retrieveProperties();
        if ($ret == 0) {
            raslogv(VCLOG_WARNING, "SKIPPING DVS ".$dvs_mor);
            continue;
        }
        $dvs_nn = $dvs->getNN();
        raslogv(VCLOG_DEBUG,"Processing DVS  $dvs_nn");
        $dbh->addObj($dvs_mor, $dvs_nn);

        //$ret  = $dvs->retrieveProperties("config");
        // Uplink port group
        $up_mor = $dvs->getUplinkPortGroupMOR();
        unset($attr); unset($val);
        $attr[] = "name";      $val[] = $dvs_nn;
        $attr[] = "pgroup_nn"; $val[] = $up_mor;  //ZZZ
        $attr[] = "pgroup_id"; $val[] = $up_mor;
        $attr[] = "pgtype";    $val[] = "uplink";
        $dbh->setAttr($dvs_mor, $attr, $val);
        
        // Other port groups ZZZ
        // Actually not possible to do!
        // Moving on ...

        // Hosts connected to this dvSwitch
        $dvs_hosts = $dvs->getHosts();
        if (empty($dvs_hosts)) continue;
        $dbh->createRelation("hosts-dvs");

        // Build array for addRelations dvs ->* host
        unset($dvs_host_mors);
        foreach ($dvs_hosts as $dvs_host) {
                $dvs_host_mor = getDvsHostMOR($dvs_host);
                $dvs_host_mors[] = $dvs_host_mor;
        }
        $dbh->addRelations("hosts-dvs", $dvs_host_mors, $dvs_mor);
        
        // Host <-> PNIC
        $dbh->createRelation("dvs-host-pnic", "non-unique");
        foreach ($dvs_hosts as $dvs_host) {
                unset($f);
                $dvs_host_mor = getDvsHostMOR($dvs_host);
                $dvs_pnic     = getDvsHostPnics($dvs_host);

                $dvs_pnic_dev = $dvs_pnic['pnicDevice'];
                if (empty($dvs_pnic_dev)) {
                    if (empty($dvs_pnic)) continue;
                    foreach ($dvs_pnic as $pnic) {
                        unset($pnic_dev);
                        $pnic_dev = $pnic['pnicDevice'];
                        parseAvmnic($pnic_dev, $dvs_mor, $dvs_host_mor, $dvs_nn, $dbh);
                    }
                }else {
                    unset($pnic_dev);
                    $pnic_dev = $dvs_pnic_dev;
                    parseAvmnic($pnic_dev, $dvs_mor, $dvs_host_mor, $dvs_nn, $dbh);
                }
        }

}

   $time = $sw->elapsedTime();
   raslogv(VCLOG_INFO, "Parsing DVS-MOR TIME $time seconds ");
   $sw->resetAndStart();

raslogv(VCLOG_INFO, "Fetching DVPG");
$dvpg_mors    = getMORS($vc, 'DistributedVirtualPortgroup');
raslogv(VCLOG_DEBUG, $dvpg_mors);
foreach ($dvpg_mors as $dvpg_mor) {
        if(empty($dvpg_mor)) continue;
        $dvpg = new dvpg($vc, $dvpg_mor);
        $ret  = $dvpg->retrieveProperties();
        if ($ret == 0) {
            raslogv(VCLOG_WARNING, "SKIPPING DVPGS ".$dvpg_mor);
            continue;
        }
        $dvpg_nn = $dvpg->getNN();
        raslogv(VCLOG_DEBUG,"Processing DVPG  $dvpg_nn");
        $dbh->addObj($dvpg_mor, $dvpg_nn);

        unset($vlans);
        unset($vlan);
        $vlans    = $dvpg->getDefaultVlan();
        foreach($vlans as $vlan_temp) {
            $vlan = $vlan.$vlan_temp['start'] . "-". $vlan_temp['end'].",";
        }
        $dvs_mor = $dvpg->getDvSwitch();

        //$ret  = $dvpg->retrieveProperties("tag");
        $isUplink = $dvpg->isUplinkPortGroup();

        unset($attr); unset($val);
        $attr[] = "name";   $val[] = $dvpg_nn;
        $attr[] = "vlan";   $val[] = $vlan;
        $attr[] = "dvs_id"; $val[] = $dvs_mor;
        $attr[] = "pgtype";    $val[] = $isUplink;

        $dbh->setAttr($dvpg_mor, $attr, $val);
}

   $time = $sw->elapsedTime();
   raslogv(VCLOG_INFO, "Parsing DVPG-MOR TIME $time seconds ");
   $sw->resetAndStart();

raslogv(VCLOG_INFO, "Fetching VMS");
$vm_mors      = getMORS($vc, 'VirtualMachine');
raslogv(VCLOG_DEBUG, $vm_mors);
foreach ($vm_mors as $vm_mor) {
        if(empty($vm_mor)) continue;
        $vm = new vm($vc, $vm_mor);

        $ret  = $vm->retrieveProperties();
        if ($ret == 0) {
            raslogv(VCLOG_WARNING, "SKIPPING VM ".$vm_mor);
            continue;
        }
        
        $vm_nn = $vm->getNN();
        raslogv(VCLOG_DEBUG,"Processing VM Details  $vm_nn");
        if ($vm->isTemplate() == "true") {
                // delete this entry from the vm table
                $dbh->deleteObj($vm_mor);
                continue;
        }
        // columns id and host_id are already populated
        unset($attr); unset($val);
        $attr[] = "name";    $val[] = $vm_nn;
        $dbh->setAttr($vm_mor, $attr, $val);
        $vm_rows = $dbh->getTableObjs("vms","id='$vm_mor'");
        $host_mor = $vm_rows[0]['host_id'];
        //populate vnics table
        unset($vnics_name); unset($vnics_mac);
        unset($vnics_pg_id); unset($vnics_pg_name);
        //$ret  = $vm->retrieveProperties("config.hardware.device");
        $vnics_name = $vm->getVnicsName();
        $vnics_mac = $vm->getVnicsMac();
        $vnics_pg_id = $vm->getVnicsPgMor();
        $vnics_pg_name = $vm->getVnicsPgName();

        if (!empty($vnics_name)) {
            for ($i = 0; $i < count($vnics_name); $i++) {
                $vnic_mor = "vnic-".$vnics_mac[$i];
                $vnic_nn = $vnics_name[$i];
                $dbh->addObj($vnic_mor, $vnic_nn);
                unset($attr); unset($val);
                $attr[] = "mac";     $val[] = $vnics_mac[$i];
                $pg_id= $vnics_pg_id[$i];
                raslogv(VCLOG_DEBUG,"Processing VNIC $vnic_mor");
                if (strcmp($vnics_pg_name[$i], "@") == 0) {
                    //for vnics on dvs, we have to query this value from
                    //dvportgroups table
                    //for vnics on vswitch we already have this value
                    $pg_row = $dbh->getTableObjs("dvportgroups", "id='$pg_id'");
                    if (empty($pg_row)) {
                        $attr[] = "dvpg_id"; $val[] = '';
                        $attr[] = "dvpg_nn"; $val[] = '';
                    }else {
                        $attr[] = "dvpg_id"; $val[] = $vnics_pg_id[$i];
                        $vnics_pg_name[$i] = $pg_row[0]['name'];
                        $attr[] = "dvpg_nn"; $val[] = $vnics_pg_name[$i];
                    } 
                }else {
                // this is a standard portgroup, check if the poirtgroup is not deleted
                    $pg_key   =$host_mor."-".$pg_id;
                    $pg_mor = "portgroup-$pg_key";
                    $pg_row = $dbh->getTableObjs("portgroups", "id='$pg_mor'");
                    if (empty($pg_row)) {
                        $attr[] = "dvpg_id"; $val[] = '';
                        $attr[] = "dvpg_nn"; $val[] = '';
                    }else {
                        $attr[] = "dvpg_id"; $val[] = $pg_mor;
                        $attr[] = "dvpg_nn"; $val[] = $vnics_pg_name[$i];
                    } 
                }

                $attr[] = "vm_id";   $val[] = $vm_mor;
                $dbh->setAttr($vnic_mor, $attr, $val);
            }
        }
}

   $time = $sw->elapsedTime();
   raslogv(VCLOG_INFO, "Parsing VM-MOR TIME $time seconds ");
   $sw->resetAndStart();


foreach ($dvs_mors as $dvs_mor) {
        if(empty($dvs_mor)) continue;
        $dvs_row   = $dbh->getTableObjs("dvss", "id='$dvs_mor'");
        $pgroup_id = $dvs_row[0]['pgroup_id'];
        $dvpg_row  = $dbh->getTableObjs("dvportgroups", "id='$pgroup_id'");
        $pgroup_nn = $dvpg_row[0]['name'];

        unset($attr); unset($val);        
        $attr[] = "pgroup_nn"; $val[] = $pgroup_nn;
        $dbh->setAttr($dvs_mor, $attr, $val);
        
}

   $time = $sw->elapsedTime();
   raslogv(VCLOG_INFO, "Parsing DVS2-MOR TIME $time seconds ");
   $sw->resetAndStart();

foreach ($dvpg_mors as $dvpg_mor) {
        if(empty($dvpg_mor)) continue;
        $dvpg_row = $dbh->getTableObjs("dvportgroups", "id='$dvpg_mor'");
        $dvs_id   = $dvpg_row[0]['dvs_id'];
        $dvs_row  = $dbh->getTableObjs("dvss", "id='$dvs_id'");
        $dvs_nn   = $dvs_row[0]['name'];

        unset($attr); unset($val);        
        $attr[] = "dvs_nn"; $val[] = $dvs_nn;
        $dbh->setAttr($dvpg_mor, $attr, $val);

// VNICs table upation of dvportgroups name for VMKernel MAC.
        $vmk_pg_id = $dvpg_row[0]['id'];
        $vmk_pg_nn = $dvpg_row[0]['name'];
        $vmk_rows = $dbh->getTableObjs("vnics", "dvpg_id='$vmk_pg_id'");
        foreach ($vmk_rows as $vmk_row) {
            if(empty($vmk_row)) continue;
            
            $vmk_pg_nn_old = $vmk_row['dvpg_nn'];
            $vmk_vm_id = $vmk_row['vm_id'];
            if(empty($vmk_pg_nn_old) && (!strcmp($vmk_vm_id,"VMKernel"))) {
                $vmk_mor = $vmk_row['id'];
                unset($attr); unset($val);
                $attr[] = "dvpg_nn"; $val[] = $vmk_pg_nn;
                $dbh->setAttr($vmk_mor, $attr, $val);
            }
        }
}

   $time = $sw->elapsedTime();
   raslogv(VCLOG_INFO, "Parsing DVPG2-MOR TIME $time seconds ");
   $sw->resetAndStart();

raslogv(RASLOG_INFO, "END: discover virtual assets from vCenter @ $url");
   $time = $csw->elapsedTime();
   raslogv(VCLOG_CRITICAL, "VC-Discover Completed ($url) in $time seconds");

   $log->lclose();
   exit(SUCCESS);

}catch (Exception $e) {
    raslogv(RASLOG_ERROR, "Error parsing the vcenter inventory ".$e->getMessage());
    raslogv(RASLOG_ERROR, $e);
    $log->lclose();
    exit(GENERIC_ERROR);
}
    function parseAvmnic($pnic_dev, $dvs_mor, $dvs_host_mor, $dvs_nn, $dbh)
    {
        raslogv(VCLOG_DEBUG, "ParseAvmnic  $pnic_dev, $dvs_mor, $dvs_host_mor, $dvs_nn");
        $f[] = $dvs_mor;
        $f[] = $dvs_host_mor;
        $f[] = $pnic_dev;
        $dbh->addRelation("dvs-host-pnic", $f);
        if (empty($pnic_dev)) continue;
        unset($attr); unset($val);
        $attr[] = "dvs";   $val[] = $dvs_mor;
        $attr[] = "dvs_nn";   $val[] = $dvs_nn;
        $whereclause = "name='".$pnic_dev."' and host_id='".$dvs_host_mor."'";
        $dbh->updateAttr("vmnics", $whereclause, $attr, $val);
    }

    function parseAPortgroup($host_nn,$host_mor, $pg, $dbh)
    {
        $pg_key   =$host_mor."-".$pg['key'];
        raslogv(VCLOG_DEBUG, "ParseAPortgroup  $host_nn,$host_mor, $pg_key");
        //$pg_key   =$pg['key'];
        $pg_vs    = $pg['vswitch'];
        $pg_nn    = $pg['spec']['name'];
        //vcenter does not allow a range for portgroup
        //but we have to save the vlan as a range
        $pg_vlan  = $pg['spec']['vlanId']. "-". $pg['spec']['vlanId'].",";
        $pg_vs_nn = $pg['spec']['vswitchName'];
        //$pg_mor = $pg_key;
        $pg_mor = "portgroup-$pg_key";
        $dbh->addObj($pg_mor, $pg_nn, "");
        //$dbh->addObj($pg_key, $pg_nn, "");

        unset($attr); unset($val);
        //$attr[] = "name";   $val[] = urldecode($pg_nn);
        $attr[] = "name";   $val[] = $pg_nn;
        $attr[] = "vs_nn";  $val[] = $pg_vs_nn;
        $attr[] = "vlan";   $val[] = $pg_vlan;
        $attr[] = "vs_id";  $val[] = $vswitch;
        $attr[] = "host_id";  $val[] = $host_mor;
        $attr[] = "host_nn";  $val[] = $host_nn;
        $dbh->setAttr($pg_mor, $attr, $val);
    }
?>
