<?php
require_once("switch/ns.php");
require_once("switch/logconfig.php");
require_once("switch/logging.php");

class portprofile {

        var $name;
        var $vlan;
        var $qos;
        var $security;
        var $mor;
        var $path;
        var $gpath;
        var $ns;

        function portprofile($name, $vlan = "", $qos = "", $security = "")
        {
                $this->name     = $name;
                $this->mor      = $mor;
                $this->vlan     = $vlan;
                $this->qos      = $qos;
                $this->security = $security;
                $this->path    = "/appm:port-profile" . "{" . $name . "}";
                $this->gpath   = "/appm:port-profile-global/port-profile" . "{" . $name . "}";
                $this->ns      = appm__ns;
                return $this;

        }
        function create($sock)
        {
                return $this->maapiCreatePP($sock);
        }
        function delete($sock)
        {
                return $this->maapiDeletePP($sock);
        }
        function maapiCreatePP($sock)
        {
                $path = $this->path;
                $ns   = $this->ns;
                $tid = maapi_start($sock, $ns);
                $rc   = maapi_create($sock, $path);
                $rc   = maapi_apply($sock);
                maapi_finish($sock);
                $msg = "Created port-profile: $this->name $rc";
                raslogv(VCLOG_INFO, $msg);

        }
        function maapiDeletePP($sock)
        {
                $path = $this->path;
                $ns   = $this->ns;

                $this->deactivatePortProfile($sock);

                $tid = maapi_start($sock, $ns);
                $rc   = maapi_delete($sock, $path);
                $rc   = maapi_apply($sock);
                maapi_finish($sock);
                $msg = "Deleting port-profile: $this->name";
                 raslogv(VCLOG_INFO, $msg);

        }
        function addStaticMac($sock, $mac)
        {

                $path = $this->gpath;
                $ns   = $this->ns;

                $path .= "/static";
                $path .= "{" . $mac . "}";
                $rc    = maapi_create($sock, $path);
                $msg = "Add: static MAC($mac) -> PP($this->name)";
                if ($rc == 0) {
                         raslogv(VCLOG_INFO, $msg . " (success)");
                } else if ($rc == 1) {
                         raslogv(VCLOG_INFO, $msg . " (already)");
                } else {
                         raslogv(VCLOG_CRITICAL, $msg . " (failed rc=$rc)");
                }
                return $rc;
        }
        function delStaticMac($sock, $mac)
        {
                $msg = "Del: static MAC: $mac -> PP: $this->name";

                $path = $this->gpath;
                $ns   = $this->ns;

                $path .= "/static";
                $path .= "{" . $mac . "}";
                
                //$path .= "/mac-address";
                //$path .= "{" . $mac . "}";

                //echo $path . "\n";

                $msg = "Delete: static MAC($mac) -> port-profile($this->name)";
                $tid = maapi_start($sock, $ns);
                $rc    = maapi_delete($sock, $path);
                $rc = maapi_apply($sock);
                if ($rc == 0) {
                         raslogv(VCLOG_INFO, $msg . " (success)");
                } else if ($rc == 1) {
                         raslogv(VCLOG_INFO, $msg . " (already)");
                } else {
                         raslogv(VCLOG_CRITICAL, $msg . " (failed rc=$rc)");
                }
                maapi_finish($sock);
                return $rc;
        }
        function activatePortProfile($sock)
        {
                $path = $this->gpath;
                $ns   = $this->ns;

                $path .= "/activate";
                $tid = maapi_start($sock, $ns);
                $rc   = maapi_create($sock, $path);
                $msg =  "Activate port-profile($this->name)";

                $rc = maapi_apply($sock);
                if ($rc == 0) {
                         raslogv(VCLOG_INFO, $msg . " (success)");
                } else {
                         raslogv(VCLOG_CRITICAL, $msg . " (failed rc=$rc)");
                }
                $rc = maapi_finish($sock);;
                return $rc;

        }
        function deactivatePortProfile($sock)
        {
                $path = $this->gpath;
                $ns   = $this->ns;

                $path .= "/activate";

                $tid = maapi_start($sock, $ns);
                $rc    = maapi_delete($sock, $path);
                $rc = maapi_apply($sock);
                $rc = maapi_finish($sock);;

                $msg = "De-activate port-profile($this->name)";
                 raslogv(VCLOG_INFO, $msg);


        }
        function setVlanProfile($sock, $switchport, $mode = "", $vlan_start = 0, $vlan_stop = 0)
        {
                /* possible values for mode vlan_start vlan_stop:
                 * access   1   1 =====> this is the case where only vlan 0 is configured on pp
                 * trunk    -1  -1 =====> this is to remove all vlans
                 * trunk    4095    4095 ====> add "switchport trunk allowed vlan all" plus 
                 *   "switchport trunk native-vlan 1"
                 * trunk    0   0 =====> add "switchport trunk native-vlan 1"
                 * trunk    0   5 =====> add "switchport trunk allowed vlan 2-5" plus 
                 *   "switchport trunk native-vlan 1"
                 * trunk    x   y =====> add "switchport trunk allowed vlan x-y"
                 */
                $path = $this->path;
                $ns   = $this->ns;
                raslogv(VCLOG_DEBUG,  "setVlanProfle $switchport, $mode, $vlan_start, $vlan_stop   PATH=$path");


                if ($switchport == "switchport") {

                        $tid = maapi_start($sock, $ns);
                        $rc = maapi_create($sock,    $path . "/vlan-profile");
                        $rc = maapi_apply($sock);
                        $rc = maapi_finish($sock);;

                        $tid = maapi_start($sock, $ns);
                        $rc = maapi_create($sock,    $path . "/vlan-profile/switchport");
                        $rc = maapi_apply($sock);
                        $rc = maapi_finish($sock);;

                        if ($vlan_start == -1 && $vlan_stop == -1) {
                                $vlan = "";
                        } else {
                                if ($vlan_start < $vlan_stop) {
                                        $vlan = $vlan_start . "-" . $vlan_stop;
                                } else {
                                        $vlan = $vlan_start;
                                }
                        }
                        raslogv(VCLOG_DEBUG,  "vlan:$vlan");

                        if ($mode == "access") {
                                $mode = $path . "/vlan-profile/switchport/mode/vlan-mode";
                                $tid = maapi_start($sock, $ns);
                                $rc = maapi_set_elem2($sock,  $mode, "access");
                                $rc = maapi_apply($sock);
                                $rc = maapi_finish($sock);
                                if (!isset($vlan)) {
                                    $tid = maapi_start($sock, $ns);
                                    $rc = maapi_set_elem2($sock,
                                                          $path . "/vlan-profile/switchport/access/vlan/name",
                                                          $vlan);
                                    $rc = maapi_apply($sock);
                                    $rc = maapi_finish($sock);
                                }
                        }

                        if ($mode == "trunk") {
                                $tid = maapi_start($sock, $ns);
                                $mode = $path . "/vlan-profile/switchport/mode/vlan-mode";
                                $rc = maapi_set_elem2($sock,  $mode, "trunk");
                                raslogv(VCLOG_DEBUG,  "mode/vlan-mode:$rc");
                                $rc = maapi_apply($sock);
                                raslogv(VCLOG_DEBUG,  "apply mode/vlan-mode:$rc");
                                $rc = maapi_finish($sock);
                                if ($vlan != "") {
                                    if ($vlan_start == 4095) {
                                        $tid = maapi_start($sock, $ns);
                                        $rc = maapi_create($sock, $path .
                                            "/vlan-profile/switchport/trunk/allowed/vlan/all");
                                        raslogv(VCLOG_DEBUG,  "trunk/allowed/vlan/all:$rc");
                                        $rc = maapi_apply($sock);
                                        $rc = maapi_finish($sock);
                                        $tid = maapi_start($sock, $ns);
                                        $rc = maapi_set_elem2($sock, $path .
                                            "/vlan-profile/switchport/trunk/native-vlan", 1);
                                        raslogv(VCLOG_DEBUG,  "trunk/native-vlan:$rc");
                                        
                                    } else if ($vlan_start == 0) {
                                        if ($vlan_stop >= 2) {
                                            unset($vlan);
                                            if ($vlan_stop == 2) {
                                                $vlan = $vlan_stop;
                                            } else {
                                                $vlan = "2-" . $vlan_stop;
                                            }
                                            $tid = maapi_start($sock, $ns);
                                            $rc = maapi_set_elem2($sock,
                                                $path . "/vlan-profile/switchport/trunk/allowed/vlan/add",
                                                $vlan);
                                            raslogv(VCLOG_DEBUG,  "trunk/allowed/vlan/add:$rc");
                                            $rc = maapi_apply($sock);
                                            $rc = maapi_finish($sock);
                                        }
                                        $tid = maapi_start($sock, $ns);
                                        $rc = maapi_set_elem2($sock, $path .
                                            "/vlan-profile/switchport/trunk/native-vlan", 1);
                                        raslogv(VCLOG_DEBUG,  "trunk/native-vlan:$rc");
                                    } else if ($vlan_start == 1) {
                                        if ($vlan_stop > 1) {
                                            raslogv(VCLOG_DEBUG,  "skipping vlan 1 in a range");
                                            unset($vlan);
                                            if ($vlan_stop == 2) {
                                                $vlan = $vlan_stop;
                                            } else {
                                                $vlan = "2-" . $vlan_stop;
                                            }
                                            $tid = maapi_start($sock, $ns);
                                            $rc = maapi_set_elem2($sock,
                                            $path . "/vlan-profile/switchport/trunk/allowed/vlan/add",
                                            $vlan);
                                        } else {
                                            raslogv(VCLOG_DEBUG,  "can not configure vlan 1 on trunk port");
                                        }
                                    } else {
                                        $tid = maapi_start($sock, $ns);
                                        $rc = maapi_set_elem2($sock,
                                            $path . "/vlan-profile/switchport/trunk/allowed/vlan/add",
                                            $vlan);
                                        raslogv(VCLOG_DEBUG,  "trunk/allowed/vlan/add:$rc");
                                    }
                                } else {
                                    $tid = maapi_start($sock, $ns);
                                    $rc = maapi_create($sock, $path . 
                                        "/vlan-profile/switchport/trunk/allowed/vlan/none");    
                                    raslogv(VCLOG_DEBUG,  "trunk/allowed/vlan/none:$rc");
                                    $rc = maapi_apply($sock);
                                    $rc = maapi_finish($sock);;
                                    $tid = maapi_start($sock, $ns);
                                    $rc = maapi_delete($sock, $path .
                                            "/vlan-profile/switchport/trunk/native-vlan");

                                }
                                $rc = maapi_apply($sock);
                                $rc = maapi_finish($sock);;
                        }
                }
                return $rc;
       }
};


$test = 0;

if ($test == 1) {

        //$vlan[start] = $start;
        //$vlan[stop]  = $stop;
        $pp = new portprofile("h4");
        $path = $pp->path;
        $ns   = $pp->ns;
        $sock = maapi_connect1();
        echo "sock $sock\n";
        $tid = maapi_start($sock, $ns);
        echo "tid $tid\n";
        $rc   = maapi_create($sock, $path);
        echo "create $rc\n";
        sleep(10);
        $rc   = maapi_apply($sock);        
        echo "apply $rc\n";
        $rc = maapi_finish($sock);
        echo "finish $rc\n";

        $pp1 = new portprofile("h5");
        $path = $pp1->path;
        $tid = maapi_start($sock, $ns);
        echo "tid $tid\n";
        $rc   = maapi_create($sock, $path);
        echo "create $rc\n";
        sleep(5);
        $rc   = maapi_apply($sock);
        echo "apply $rc\n";
        $rc = maapi_finish($sock);
        echo "finish $rc\n";
        maapi_close($sock);

        //$pp->maapiCreatePP();
        //$pp->setVlanProfile("switchport", "trunk", 3, 9);        //$pp->setVlanProfile("switchport", "access", 7);
        //$pp->activatePortProfile();
        //$pp->deactivatePortProfile();

}
